/*
Head 2 degrees of freedom + sharp ranger
Hardware: 2x microservo, SHARP_GP2D_12(10-80cm)
Horizzontal     - is left(ccw)(limit -80)    + is right(cw)(limit +80)
Vertical       - is down          + is up

Array Map: vertical: -70 -60 -50  0     horizzontal: -80 to +80 step 10
*/


//pinout
const int pinSharp=5;
const int pinServoV=3;
const int pinServoH=7;


//head alignment parameters
const int servoVZero=73;// - is down   + is up
const int servoHZero=80;// - is left + is right
const float servoVRatio=1.15;
const float servoHRatioPos=1;
const float servoHRatioNeg=1.2;


//sharp parameters
const int sharpF70=312;const int sharpF70Err=27;//ring0
const int sharpF60=283;const int sharpF60Err=30;//ring1
const int sharpF50=236;const int sharpF50Err=32;//ring2
const int sharpF40=194;const int sharpF40Err=35;//not used


//servo setup
Servo servoV;Servo servoH;

//head status
int posH=0;int posV=0;


void headInit(){
  servoV.attach(pinServoV);
  servoH.attach(pinServoH);
  headSetV(0);
  headSetH(0);  
}


int sharpGetRaw(){
  int a=analogRead(pinSharp);
  return a;
}

int sharpGetCm(){//only for flat surface
  int a=analogRead(pinSharp);
  if (a>508){return 0;}
  else if (a>276) {return 20-(a-276)/23 ;}
  else if (a>190) {return 30-(a-190)/9 ;}
  else if (a>150) {return 40-(a-150)/4 ;}
  else if (a>115) {return 50;}
  else {return 80;}
}


 
 
void refreshRing0(int fromC, int toC){//rinfresca l'anello 0  a -70 gradi
  boolean st=true;
  int leftCount=0;int rightCount=0;
  ringCollision[0]=false;
  if ( abs(posH-(-80+fromC*10))<abs(posH-(-80+toC*10)) ){
       for (int h=fromC;h<=toC;h++){
         int hp=-80+h*10;
         headGoto(hp,-70);if (st){delay (60);st=false;}
         int r=sharpGetRaw();   
         if (abs(sharpF70-r)>sharpF70Err){
           rangeMap[0][h]=-1; buzzerPlay(5000,10);
           if (h<8){leftCount++;}else if(h>8){rightCount++;}
           if (h>=3&&h<=13){ringCollision[0]=true;}
         }else{rangeMap[0][h]=abs(sharpF70-r);}
       }
  }else{
         for (int h=toC;h>=fromC;h--){
         int hp=-80+h*10;
         headGoto(hp,-70);if (st){delay (60);st=false;}
         int r=sharpGetRaw();   
         if (abs(sharpF70-r)>sharpF70Err){
             rangeMap[0][h]=-1;buzzerPlay(5000,10);
             if (h<8){leftCount++;}else if(h>8){rightCount++;}
             if (h>=3&&h<=13){ringCollision[0]=true;}
         }else{rangeMap[0][h]=abs(sharpF70-r);}
       }
  }
  
  if (leftCount>rightCount){ringSide[0]='L';}else{ringSide[0]='R';}
}
 
 
 
void refreshRing1(int fromC, int toC){//rinfresca l'anello 1  a -60 gradi
  boolean st=true;
  int leftCount=0;int rightCount=0;
  ringCollision[1]=false;
  if ( abs(posH-(-80+fromC*10))<abs(posH-(-80+toC*10)) ){
       for (int h=fromC;h<=toC;h++){
         int hp=-80+h*10;
         headGoto(hp,-60);if (st){delay (60);st=false;}
         int r=sharpGetRaw();   
         if (abs(sharpF60-r)>sharpF60Err){
           rangeMap[1][h]=-1;buzzerPlay(5000,10);
           if (h<8){leftCount++;}else if(h>8){rightCount++;}  
           if (h>=3&&h<=12){ringCollision[1]=true;}         
         }else{rangeMap[1][h]=abs(sharpF60-r);}
       }
  }else{
         for (int h=toC;h>=fromC;h--){
         int hp=-80+h*10;
         headGoto(hp,-60);if (st){delay (60);st=false;}
         int r=sharpGetRaw();   
         if (abs(sharpF60-r)>sharpF60Err){
           rangeMap[1][h]=-1;buzzerPlay(5000,10);
           if (h<8){leftCount++;}else if(h>8){rightCount++;}    
           if (h>=3&&h<=12){ringCollision[1]=true;}           
         }else{rangeMap[1][h]=abs(sharpF60-r);}
       }
  }
  if (leftCount>rightCount){ringSide[1]='L';}else{ringSide[1]='R';}
} 
 
 
 
 
void refreshRing2(int fromC, int toC){//rinfresca l'anello 2  a -50 gradi
  boolean st=true;
  int leftCount=0;int rightCount=0;
  ringCollision[2]=false;
  if ( abs(posH-(-80+fromC*10))<abs(posH-(-80+toC*10)) ){
       for (int h=fromC;h<=toC;h++){
         int hp=-80+h*10;
         headGoto(hp,-50);if (st){delay (60);st=false;}
         int r=sharpGetRaw();   
         if (abs(sharpF50-r)>sharpF50Err){
           rangeMap[2][h]=-1;buzzerPlay(5000,10);
           if (h<8){leftCount++;}else if(h>8){rightCount++;}   
           if (h>=5&&h<=10){ringCollision[2]=true;}             
         }else{rangeMap[2][h]=abs(sharpF50-r);}
       }
  }else{
         for (int h=toC;h>=fromC;h--){
         int hp=-80+h*10;
         headGoto(hp,-50);if (st){delay (60);st=false;}
         int r=sharpGetRaw();   
         if (abs(sharpF50-r)>sharpF50Err){  
           rangeMap[2][h]=-1;buzzerPlay(5000,10);
           if (h<8){leftCount++;}else if(h>8){rightCount++;}              
           if (h>=5&&h<=10){ringCollision[2]=true;}             
         }else{rangeMap[2][h]=abs(sharpF50-r);}
       }
  }
  if (leftCount>rightCount){ringSide[2]='L';}else{ringSide[2]='R';}
} 
 
 
void refreshRing3(int fromC, int toC){//rinfresca l'anello 3  a zero gradi
  boolean st=true;
  int leftCount=0;int rightCount=0;
  ringCollision[3]=false;
  if ( abs(posH-(-80+fromC*10))<abs(posH-(-80+toC*10)) ){
       for (int h=fromC;h<=toC;h++){
         int hp=-80+h*10;
         headGoto(hp,0);if (st){delay (60);st=false;}
         int r=sharpGetCm();  
         if (r<30){
           rangeMap[3][h]=-1;buzzerPlay(5000,10);
           if (h<8){leftCount++;}else if(h>8){rightCount++;}  
           if (h>=5&&h<=10){ringCollision[3]=true;}          
         }else{rangeMap[3][h]=r;}
       }
  }else{
         for (int h=toC;h>=fromC;h--){
         int hp=-80+h*10;
         headGoto(hp,0);if (st){delay (60);st=false;}
         int r=sharpGetCm();  
         if (r<30){
           rangeMap[3][h]=-1;buzzerPlay(5000,10);
           if (h<8){leftCount++;}else if(h>8){rightCount++;}            
           if (h>=5&&h<=10){ringCollision[3]=true;}
         }else{rangeMap[3][h]=r;}
       }
  }
  if (leftCount>rightCount){ringSide[3]='L';}else{ringSide[3]='R';}
}  
  
  


  
void rangeMapAll(){
   refreshRing0(0,16);
   refreshRing1(0,16);
   refreshRing2(0,16);
   refreshRing3(0,16);   
   headSetH(0);headSetV(0);   
}



boolean alarmFront(){
 refreshRing0(3,11);
 refreshRing1(2,12);
 refreshRing2(6,10);
 refreshRing3(6,10);
 headSetH(0);headSetV(0);
}
















//mecha routines
void headSetV(int pos){// - is down   + is up
  posV=pos;
  pos=pos*servoVRatio+servoVZero;
  servoV.write(pos); 
  
}
void headSetH(int pos){// + is right - is left
  posH=pos;
  if (pos>=0){pos=-pos*servoHRatioPos+servoHZero;}else{pos=-pos*servoHRatioNeg+servoHZero;}
  servoH.write(pos);
}


void headGoto(int trgH, int trgV){
  while ( (posH!=trgH)||(posV!=trgV) ){
    if(posH!=trgH){
      int stepH=(trgH-posH)/abs (trgH-posH);
      headSetH(posH+stepH);
    }
    
    if(posV!=trgV){
      int stepV=(trgV-posV)/abs (trgV-posV);
      headSetV(posV+stepV);
      delay(2);//extra delay for vertical slower movement
    }
    delay(3);
  }
  delay(35);//final delay per smorzare le oscillazioni sul punto di arrivo
}


